*
* Replication for Blanchard and Quah(1989), "The Dynamic Effects of Aggregate
* Demand and Supply Disturbances", AER, vol 79, no. 4, pp 655-673. The data set
* here is (very) slightly different from the one used in the paper. The difference
* is not apparent in any of the graphs, but Table I doesn't quite match.
*
* This includes everything except the figures and tables that require simulations.
*
cal(q) 1948
open data bqdata.xls
data(format=xls,org=cols) 1948:1 1987:4 gnp gd87 lhmur
*
set loggnp = log(gnp)
set loggd  = log(gd87)
set logrgnp = loggnp-loggd+log(100)
set dlogrgnp = 100.0*(logrgnp-logrgnp{1})
*
* Extract separate means from the GNP growth series. Save the fitted values for
* rebuilding the data later.
*
set dummy1 = t<=1973:4
set dummy2 = t>1973:4
linreg dlogrgnp
# dummy1 dummy2
set gdpadjust = %resids
prj means_from_gnp
*
* Remove a linear trend from unemployment
*
filter(remove=trend) lhmur / uradjust
set trend_from_ur = lhmur-uradjust
*
system(model=bqmodel)
var gdpadjust uradjust
lags 1 to 8
det constant
end(system)
*
estimate(noprint,resids=resids)
*
* Do the standard B-Q factor. By construction, this will have a long run loading
* of zero from the second shock ("demand") to the first dependent variable.
*
compute factor=%bqfactor(%sigma,%varlagsums)
*
* The zero restriction only determines the shape, not the sign, of the demand
* shock. If we want it defined so that the contemporaneous response of output is
* positive (that will be the (1,2) element in the factor), we need to flip the
* sign of the second column if the response has the wrong sign.
*
{
if factor(1,2)<0.0
   compute factor=factor*%diag(||1.0,-1.0||)
}
*
* Figures 1 and 2. Responses of output (1st variable) are accumulated.
*
@varirf(model=bqmodel,decomp=factor,steps=40,byshock,noscale,$
  vlabels=||"Output","Unemployment"||,labels=||"Supply","Demand"||,accum=||1||)
*
* Do the historical decomposition. The GNP components are for the (separately
* demeaned) growth rates, while the UR components are for the detrended data. In
* order to get the actual data back, we need to first add the removed
* deterministic components to the base forecast, and then accumulate all the GNP
* components. Those also need to be scaled by .01 to put them back into the
* correct scale for log GNP. Finally, to complete the "undifferencing", we need to
* add on to the base forecast the final value of log GNP before the forecast
* period.
*
history(model=bqmodel,factor=factor,results=histdecomp)
*
* Put the removed means back into the GNP growth, and the removed trend back into
* unemployment.
*
set histdecomp(1,1) = histdecomp(1,1)+means_from_gnp
set histdecomp(1,2) = histdecomp(1,2)+trend_from_ur
*
* Accumulate the GNP components and scale them back by .01
*
do j=1,3
   acc histdecomp(j,1)
   set histdecomp(j,1) = histdecomp(j,1)*.01
end do j
*
* Add back into the forecast component the final value of log gnp before the
* forecast period.
*
set histdecomp(1,1) = histdecomp(1,1)+logrgnp(1950:1)
*
* Add the base + effects of the supply shock to get the output without the demand
* shock.
*
set lessdemand = histdecomp(1,1)+histdecomp(2,1)
graph(footer="Figure 7. Output Fluctuations Absent Demand")
# lessdemand
*
@NBERCycles(peaks=peaks,trough=troughs)
*
* This gets the scale correct for the spikes showing the peaks and troughs
*
set peaks   = .10*peaks
set troughs = -.10*troughs
*
graph(footer="Figure 8. Output Fluctuations Due to Demand",$
  ovcount=2,overlay=spike,ovsame) 3
# histdecomp(3,1)
# peaks    1950:2 * 2
# troughs  1950:2 * 2
*
* Repeat for unemployment
*
set lessdemand = histdecomp(1,2)+histdecomp(2,2)
graph(footer="Figure 9. Unemployment Fluctuations Absent Demand",min=0.0)
# lessdemand
*
@NBERCycles(peaks=peaks,trough=troughs)
set peaks   = 4.0*peaks
set troughs = -4.0*troughs
*
graph(footer="Figure 10. Unemployment Fluctuations Due to Demand",$
  ovcount=2,overlay=spike,ovsame) 3
# histdecomp(3,2)
# peaks    1950:2 * 2
# troughs  1950:2 * 2
*
* Construction of Table 1
*
* Calculate structural residuals. This takes the VECT[SERIES] of VAR residuals
* (resids) and produces the VECT[SERIES] of structural residuals (sresids). As the
* factor is constructed, sresids(1) will be the supply shocks and sresids(2) the
* demand shocks.
*
@StructResids(factor=factor) resids / sresids
*
* As noted above, this is the one place where the difference between this data set
* and the one used in producing the paper is apparent.
*
report(action=define)
report(atrow=1,atcol=1,align=center) "Quarter" "Demand(Percent)" "Supply(Percent)"
do time=1973:3,1975:1
   report(row=new,atcol=1) %datelabel(time) sresids(2)(time) sresids(1)(time)
end do time
report(row=new)
do time=1979:1,1980:2
   report(row=new,atcol=1) %datelabel(time) sresids(2)(time) sresids(1)(time)
end do time
report(action=format,picture="*.#")
report(action=show)

